# -*- coding: utf-8 -*-
## This module contains all content types used in PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains all content types used in PloneArticle product
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
try:
    from Products.LinguaPlone.public import registerType, Schema
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import registerType, Schema

# Products imports
from Products.ATContentTypes.content.document import ATDocument, \
     ATDocumentSchema

from Products.PloneArticle.config import PLONEARTICLE_TOOL

from Products.PloneArticle.interfaces import IPloneArticle
from Products.PloneArticle.field import FileInnerContentField, \
     ImageInnerContentField, LinkInnerContentField
from Products.PloneArticle.widget import FileInnerContentWidget, \
    ImageInnerContentWidget, LinkInnerContentWidget
from Products.PloneArticle.content.mixin import ArticleMixin
from Products.PloneArticle.pafti import PloneArticleFactoryTypeInformation

from Products.PloneArticle.version import CURRENT_ARTICLE_VERSION

# Inherits schema from ATDocument
# Copy it to make sure PloneArticle schema is totally independant
PloneArticleSchema = ATDocumentSchema.copy() + Schema((
    FileInnerContentField(
        'files',
        searchable=True,
        schemata='files',
        widget=FileInnerContentWidget(
            label='Files',
            label_msgid='label_files',
            i18n_domain='plonearticle',
            ),
        ),
    ImageInnerContentField(
        'images',
        searchable=True,
        schemata='images',
        widget=ImageInnerContentWidget(
            label='Images',
            label_msgid='label_images',
            i18n_domain='plonearticle',
            ),
        ),
    LinkInnerContentField(
        'links',
        searchable=True,
        schemata='links',
        widget=LinkInnerContentWidget(
            label='Links',
            label_msgid='label_links',
            i18n_domain='plonearticle',
            ),
        ),
    ),)

# Inherits actions from ATDocument
# Copy them to make sure PloneArticle actions are totally independant
# PloneArticleActions = deepcopy(ATDocument.actions)

# temporary change (before changing proxies_edit_form to widget) :

PloneArticleActions = (
        {
        'id'          : 'view',
        'name'        : 'View',
        'action'      : 'string:${object_url}/',
        'permissions' : (CCP.View,),
        'visible'     : True,
         },
        {
        'id'          : 'edit',
        'name'        : 'Edit',
        'action'      : 'string:${object_url}/edit',
        'permissions' : (CCP.ModifyPortalContent,),
        'visible'     : True,
         },
        {'id'         : 'display',
         'name'       : 'Templates',
         'action'     : 'string:${object_url}/display',
         'condition'  : 'not: object/isTemporary',
         'permissions': (CCP.ModifyPortalContent,),
         'visible'    : True,
         },
        {
        'id'          : 'metadata',
        'name'        : 'Properties',
        'action'      : 'string:${object_url}/properties',
        'permissions' : (CCP.ModifyPortalContent,),
        'visible'     : True,
         },
        {
        'id'          : 'local_roles',
        'name'        : 'Sharing',
        'action'      : 'string:${object_url}/sharing',
        'permissions' : (CCP.ManageProperties,),
        'visible'     : True,
         },
        )

view_aliases = ATDocument.aliases.copy()
view_aliases['display'] = 'pa_select_model'

class PloneArticle(ArticleMixin, ATDocument):
    """A rich document containing files, images, links"""

    # Standard content type setup
    portal_type = meta_type = 'PloneArticle'
    archetype_name = 'Article'
    content_icon = 'plonearticle.gif'
    schema = PloneArticleSchema
    typeDescription = 'A rich document containing files, images, links'
    typeDescMsgId = 'description_edit_plonearticle'

    _at_fti_meta_type = PloneArticleFactoryTypeInformation.meta_type
    aliases = view_aliases

    # Set up our views - these are available from the 'display' menu
    default_view = 'pa_model1'
    immediate_view = 'pa_model1'
    suppl_views = (
        'pa_model1',
        'pa_model2',
        'pa_model3',
        'pa_model4',
        'pa_model5',
        'pa_model6',
        'pa_model7',
        'pa_model8',
        'pa_model9',
        'pa_model10',
        )

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Make sure we get all the interface declarations from ATDocument,
    # which includes support for ISelectableBrowserDefault to get the
    # 'display' menu to work, IHistoryAware and other standard interfaces.
    __implements__ = ATDocument.__implements__ +  \
        ArticleMixin.__implements__ + (IPloneArticle, )
    
    __pa_version = ()

    # Get the standard actions (tabs)
    actions = PloneArticleActions
    security = ClassSecurityInfo()

    security.declareProtected(CCP.ModifyPortalContent, 'initializeArchetype')
    def initializeArchetype(self, **kwargs):
        """called by the generated add* factory in types tool

        Overwritten to call the 'right' initializeArchetype and set PA version
        number at creation time.
        """
        ATDocument.initializeArchetype(self, **kwargs)

        self.__pa_version = CURRENT_ARTICLE_VERSION

    security.declareProtected(CCP.ManagePortal, 'getPAVersion')
    def getPAVersion(self):
        """Return the version used to create this article
        """
        return self.__pa_version

    security.declareProtected(CCP.ManagePortal, 'setPAVersion')
    def setPAVersion(self, version):
        """Set the version used to create this article. Usually called after
        migration
        """
        self.__pa_version = version

    ### ISelectableBrowserDefault overrides ###
    def getAvailableLayouts(self):
        """Get the layouts from enabled models (configured by plonarticle_tool).
        """
        tool = getToolByName(self, PLONEARTICLE_TOOL)
        models = tool.getModelRegistry()
        model_ids, default = tool.getEnabledModelsForType(self.aq_explicit.portal_type)
        result = []
        for mid in model_ids:
            model = models.get(mid, None)
            if model is None:
                continue
            result.append((mid, model.Title()))

        return result

    def canSetDefaultPage(self):
        """
        Override BrowserDefaultMixin because default page stuff doesn't make
        sense for articles.
        """
        return False
    ###########################################

registerType(PloneArticle)
