# -*- coding: utf-8 -*-
## Product description
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Monkey patches applied for plone 2.1:

- CMFDynamicViewFTI: getDefaultPage checks if default_page attr is callable, but
  calls it without acquisition. Fixed in this patch.

- PloneTool getDefaultPage: fix in order to be able to have a dynamic page
returned by the object itself. CMFDynamicView define 'default_page' can be
callable, but PloneTool does not take care. Too bad. Under Plone 2.5 we use a
'default_page' Five View (so nice).

- make some basic classes traversable (buried from CMFPlone/configure.zcml
  (2.5)). Here we'll do it programmatically.
"""
from Products.PloneArticle import LOG

from Products.CMFCore.utils import getToolByName


from Products.CMFDynamicViewFTI.fti import safe_callable, om_has_key
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation

def dynview_fti_patched_getDefaultPage(self, context, check_exists=False):
    """Get the default page from a folderish object

    Non folderish objects don't have a default view.

    If check_exists is enabled the method makes sure the object with the default
    page id exists.

    Return -- None for no default page or a string
    """
    if not getattr(aq_base(context), 'isPrincipiaFolderish', False):
        return None # non folderish objects don't have a default page per se

    default_page = getattr(aq_base(context), 'default_page', None)

    if safe_callable(default_page):
        # PATCHED here
        # default_page = default_page()
        default_page = context.default_page()
        
    if not default_page:
        return None
    if isinstance(default_page, (tuple, list)):
        default_page = default_page[0]
    if not isinstance(default_page, str):
        raise TypeError, ("default_page must be a string, got %s(%s):" %
                          (default_page, type(default_page)))

    if check_exists and not om_has_key(context, default_page):
        return None

    return default_page

DynamicViewTypeInformation.getDefaultPage = dynview_fti_patched_getDefaultPage

LOG.info("Applied patch on DynamicViewTypeInformation.getDefaultPage")

###
from Products.CMFPlone.PloneTool import PloneTool
from types import UnicodeType, StringType
from Acquisition import aq_base
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.interfaces.BrowserDefault import IBrowserDefault
from Products.CMFPlone.interfaces.BrowserDefault import IDynamicViewTypeInformation
from Products.CMFPlone.interfaces.Translatable import ITranslatable

def plone_tool_patched_getDefaultPage(self, obj):
    """Given a folderish item, find out if it has a default-page using
    the following lookup rules:

        1. A content object called 'index_html' wins
        2. If the folder implements IBrowserDefault, query this
        3. Else, look up the property default_page on the object
            - Note that in this case, the returned id may *not* be of an
              object in the folder, since it could be acquired from a
              parent folder or skin layer
        4. Else, look up the property default_page in site_properties for
            magic ids and test these

    The id of the first matching item is then used to lookup a translation
    and if found, its id is returned. If no default page is set, None is
    returned. If a non-folderish item is passed in, return None always.
    """

    def lookupTranslationId(obj, page):
        implemented = ITranslatable.isImplementedBy(obj)
        if not implemented or implemented and not obj.isTranslation():
            pageobj = getattr(obj, page, None)
            if pageobj is not None and ITranslatable.isImplementedBy(pageobj):
                translation = pageobj.getTranslation()
                if translation is not None and \
                   ids.has_key(translation.getId()):
                    page = translation.getId()
        return page

    # Short circuit if we are not looking at a Folder
    if not obj.isPrincipiaFolderish:
        return None

    # The list of ids where we look for default
    ids = {}

    portal = getToolByName(self, 'portal_url').getPortalObject()
    wftool = getToolByName(self, 'portal_workflow')

    # For BTreeFolders we just use has_key, otherwise build a dict
    if base_hasattr(obj, 'has_key'):
        ids = obj
    else:
        for id in obj.objectIds():
            ids[id] = 1

    # 1. test for contentish index_html
    if ids.has_key('index_html'):
        return lookupTranslationId(obj, 'index_html')

    # 2. Test for IBrowserDefault
    if IBrowserDefault.isImplementedBy(obj):
        fti = obj.getTypeInfo()
        if fti is not None:
            # Also check that the fti is really IDynamicViewTypeInformation
            if IDynamicViewTypeInformation.isImplementedBy(fti):
                page = fti.getDefaultPage(obj, check_exists=True)
                if page is not None:
                    return lookupTranslationId(obj, page)

    # 3. Test for default_page property in folder, then skins
    pages = getattr(aq_base(obj), 'default_page', [])
    # XXX PATCHED:
    if safe_callable(pages):
        # if 'None': use an empty list
        pages = obj.default_page() or []
    # XXX
    if type(pages) in (StringType, UnicodeType):
        pages = [pages]
    for page in pages:
        if page and ids.has_key(page):
            return lookupTranslationId(obj, page)
    for page in pages:
        if portal.unrestrictedTraverse(page, None) is not None:
            return lookupTranslationId(obj, page)

    # 4. Test for default sitewide default_page setting
    site_properties = portal.portal_properties.site_properties
    for page in site_properties.getProperty('default_page', []):
        if ids.has_key(page):
            return lookupTranslationId(obj, page)

    return None

PloneTool.getDefaultPage = plone_tool_patched_getDefaultPage

LOG.info("Applied patch on PloneTool.getDefaultPage")


## Plone 2.5 CMFPlone/configure.zcml
## We need it for registring views with for="*"
##
##   <!-- Make all CMF style tools traversable via SimpleItem and Folder
##   !danger! -->
##   <five:traversable class="OFS.SimpleItem.SimpleItem" />
##
##   <five:traversable
##       class="OFS.Folder.Folder"
##       />
##
##  from Archetypes 1.4:
##  <five:traversable class="Products.Archetypes.atapi.BaseObject" />
from Products.Five.fiveconfigure import classTraversable
from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder
from Products.Archetypes.atapi import BaseObject

classTraversable(SimpleItem)
LOG.info("made OFS.SimpleItem.SimpleItem traversable (for Five Views)")
classTraversable(Folder)
LOG.info("made OFS.Folder.Folder traversable (for Five Views)")
classTraversable(BaseObject)
LOG.info("made Products.Archetypes.atapi.BaseObject traversable (for Five Views)")
